@@ -1,5 +1,5 @@
 /*
- * $Id: gfhost.c 9191 2014-08-01 07:41:30Z m-kasahr $
+ * $Id: gfhost.c 9937 2015-11-10 17:00:48Z n-soda $
  */
 
 #include <assert.h>
@@ -1076,7 +1076,7 @@ list_all(const char *architecture, const char *domainname,
 
 	if (nhosts == 0 || ndisplayed == 0) {
 		fprintf(stderr, "%s: %s\n",  program_name,
-		    gfarm_error_string(GFARM_ERRMSG_NO_FILESYSTEM_NODE));
+		    gfarm_error_string(GFARM_ERR_NO_FILESYSTEM_NODE));
 		return (GFARM_ERR_UNKNOWN_HOST);
 	}
 
