@@ -1,5 +1,5 @@
 /*
- * $Id: gfarm_dirtree.h 9753 2015-06-24 00:13:12Z takuya-i $
+ * $Id: gfarm_dirtree.h 9867 2015-08-25 03:07:38Z takuya-i $
  */
 
 typedef struct gfarm_dirtree gfarm_dirtree_t;
@@ -25,7 +25,7 @@ typedef struct gfarm_dirtree_entry {
 } gfarm_dirtree_entry_t;
 
 gfarm_error_t gfarm_dirtree_init_fork(gfarm_dirtree_t **, GFURL, GFURL,
-	int, int, int);
+	int, int, int, int, regex_t *);
 gfarm_error_t gfarm_dirtree_open(gfarm_dirtree_t *);
 
 gfarm_error_t gfarm_dirtree_checknext(gfarm_dirtree_t *,
