@@ -1,5 +1,5 @@
 /*
- * $Id: config.c 9874 2015-08-25 08:55:50Z takuya-i $
+ * $Id: config.c 9952 2015-11-27 07:20:16Z tatebe $
  */
 
 #include <pthread.h>
@@ -99,7 +99,7 @@ gfarm_config_static_init(struct gfarm_context *ctxp)
 
 	s->config_file = GFARM_CONFIG;
 	s->shared_key_file = NULL;
-	/* xattr_cache_list is initialized in gfarm_init_config() */
+	gfarm_stringlist_init(&s->xattr_cache_list);
 	s->local_ug_maps_tab = NULL;
 	s->local_username = NULL;
 	s->local_homedir = NULL;
@@ -113,6 +113,9 @@ gfarm_config_static_init(struct gfarm_context *ctxp)
 	return (GFARM_ERR_NO_ERROR);
 }
 
+static void local_ug_maps_tab_free(void);
+static void debug_command_argv_free(void);
+
 void
 gfarm_config_static_term(struct gfarm_context *ctxp)
 {
@@ -122,17 +125,11 @@ gfarm_config_static_term(struct gfarm_context *ctxp)
 		return;
 
 	free(s->shared_key_file);
-	/*
-	 * The following gfarm_stringlist_free_deeply() call also is
-	 * performed by gfarm_free_config(), but the repeated call of
-	 * this function has no problem.
-	 */
 	gfarm_stringlist_free_deeply(&s->xattr_cache_list);
-	if (s->local_ug_maps_tab != NULL)
-		gfarm_hash_table_free(s->local_ug_maps_tab);
+	local_ug_maps_tab_free();
 	free(s->local_username);
 	free(s->local_homedir);
-	free(s->debug_command_argv);
+	debug_command_argv_free();
 	free(s->argv0);
 	free(s);
 }
@@ -375,6 +372,7 @@ local_ug_maps_tab_free(void)
 		gfarm_stringlist_free_deeply(&ugm->local_group_map_file_list);
 		gfarm_hash_iterator_purge(&it);
 	}
+	gfarm_hash_table_free(staticp->local_ug_maps_tab);
 }
 
 static struct gfarm_local_ug_maps *
@@ -846,6 +844,7 @@ gfarm_set_local_user_for_this_uid(uid_t uid)
 /* GFS dependent */
 int gfarm_spool_server_listen_backlog = GFARM_CONFIG_MISC_DEFAULT;
 char *gfarm_spool_server_listen_address = NULL;
+int gfarm_spool_server_back_channel_rcvbuf_limit = GFARM_CONFIG_MISC_DEFAULT;
 char *gfarm_spool_root = NULL;
 static struct {
 	enum gfarm_spool_check_level level;
@@ -911,6 +910,7 @@ int gfarm_iostat_max_client = GFARM_CONFIG_MISC_DEFAULT;
 /* miscellaneous */
 #define GFARM_LOG_MESSAGE_VERBOSE_DEFAULT	0
 #define GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT 30 /* 30 seconds */
+#define GFARM_GFMD_AUTHENTICATION_TIMEOUT_DEFAULT 10 /* 10 seconds */
 #define GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT 30 /* 30 seconds */
 #define GFARM_GFSD_CONNECTION_TIMEOUT_DEFAULT 30 /* 30 seconds */
 #define GFARM_ATTR_CACHE_LIMIT_DEFAULT		40000 /* 40,000 entries */
@@ -933,6 +933,8 @@ int gfarm_iostat_max_client = GFARM_CONFIG_MISC_DEFAULT;
 #define GFARM_GFMD_CONNECTION_CACHE_DEFAULT  8 /*  8 free connections */
 #define GFARM_METADB_MAX_DESCRIPTORS_DEFAULT	(2*65536)
 #define GFARM_METADB_REPLICA_REMOVER_BY_HOST_SLEEP_TIME_DEFAULT	20000000
+#define GFARM_SOCKBUF_LIMIT_UNLIMITED GFARM_CONFIG_MISC_DEFAULT /* no limit */
+#define GFARM_SOCKBUF_LIMIT_DEFAULT		GFARM_SOCKBUF_LIMIT_UNLIMITED
 							/* nanosec. */
 #define GFARM_METADB_REPLICA_REMOVER_BY_HOST_INODE_STEP_DEFAULT	1024
 #define GFARM_CLIENT_DIGEST_CHECK_DEFAULT	0
@@ -968,6 +970,7 @@ int gfarm_metadb_thread_pool_size = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_job_queue_length = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_heartbeat_interval = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_dbq_size = GFARM_CONFIG_MISC_DEFAULT;
+int gfarm_metadb_server_back_channel_sndbuf_limit = GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_replica_remover_by_host_sleep_time =
 	GFARM_CONFIG_MISC_DEFAULT;
 int gfarm_metadb_replica_remover_by_host_inode_step =
@@ -2777,6 +2780,10 @@ parse_one_line(char *s, char *p, char **op)
 	} else if (strcmp(s, o = "spool_server_cred_name") == 0) {
 		e = parse_cred_config(p, GFS_SERVICE_TAG,
 		    gfarm_auth_server_cred_name_set);
+	} else if (strcmp(s, o = "spool_server_back_channel_rcvbuf_limit")
+	    == 0) {
+		e = parse_set_misc_int(p,
+		    &gfarm_spool_server_back_channel_rcvbuf_limit);
 	} else if (strcmp(s, o = "spool_check_level") == 0) {
 		e = parse_spool_check_level(p);
 	} else if (strcmp(s, o = "spool_base_load") == 0) {
@@ -2917,6 +2924,9 @@ parse_one_line(char *s, char *p, char **op)
 	} else if (strcmp(s, o = "no_file_system_node_timeout") == 0) {
 		e = parse_set_misc_int(
 		    p, &gfarm_ctxp->no_file_system_node_timeout);
+	} else if (strcmp(s, o = "gfmd_authentication_timeout") == 0) {
+		e = parse_set_misc_int(
+		    p, &gfarm_ctxp->gfmd_authentication_timeout);
 	} else if (strcmp(s, o = "gfmd_reconnection_timeout") == 0) {
 		e = parse_set_misc_int(
 		    p, &gfarm_ctxp->gfmd_reconnection_timeout);
@@ -3001,6 +3011,10 @@ parse_one_line(char *s, char *p, char **op)
 		e = parse_set_misc_int(p, &gfarm_metadb_heartbeat_interval);
 	} else if (strcmp(s, o = "metadb_server_dbq_size") == 0) {
 		e = parse_set_misc_int(p, &gfarm_metadb_dbq_size);
+	} else if (strcmp(s, o = "metadb_server_back_channel_sndbuf_limit")
+	    == 0) {
+		e = parse_set_misc_int(p,
+		    &gfarm_metadb_server_back_channel_sndbuf_limit);
 	} else if (strcmp(s, o = "metadb_replica_remover_by_host_sleep_time")
 	     == 0) {
 		e = parse_set_misc_int(p,
@@ -3091,22 +3105,6 @@ parse_one_line(char *s, char *p, char **op)
 	return (e);
 }
 
-gfarm_error_t
-gfarm_init_config(void)
-{
-	gfarm_stringlist_init(&staticp->xattr_cache_list);
-	return (GFARM_ERR_NO_ERROR);
-}
-
-gfarm_error_t
-gfarm_free_config(void)
-{
-	gfarm_stringlist_free_deeply(&staticp->xattr_cache_list);
-	local_ug_maps_tab_free();
-	debug_command_argv_free();
-	return (GFARM_ERR_NO_ERROR);
-}
-
 gfarm_error_t
 gfarm_config_read_file(FILE *config, int *lineno_p)
 {
@@ -3236,6 +3234,15 @@ gfarm_config_set_default_misc(void)
 	if (gfarm_metadb_server_listen_backlog == GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_metadb_server_listen_backlog = LISTEN_BACKLOG_DEFAULT;
 
+	if (gfarm_spool_server_back_channel_rcvbuf_limit ==
+	    GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_spool_server_back_channel_rcvbuf_limit =
+		    GFARM_SOCKBUF_LIMIT_DEFAULT;
+	if (gfarm_metadb_server_back_channel_sndbuf_limit ==
+	    GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_metadb_server_back_channel_sndbuf_limit =
+		    GFARM_SOCKBUF_LIMIT_DEFAULT;
+
 	if (gfarm_ctxp->log_level == GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_ctxp->log_level = GFARM_DEFAULT_PRIORITY_LEVEL_TO_LOG;
 	gflog_set_priority_level(gfarm_ctxp->log_level);
@@ -3248,6 +3255,10 @@ gfarm_config_set_default_misc(void)
 	    GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_ctxp->no_file_system_node_timeout =
 		    GFARM_NO_FILE_SYSTEM_NODE_TIMEOUT_DEFAULT;
+	if (gfarm_ctxp->gfmd_authentication_timeout ==
+	    GFARM_CONFIG_MISC_DEFAULT)
+		gfarm_ctxp->gfmd_authentication_timeout =
+		    GFARM_GFMD_AUTHENTICATION_TIMEOUT_DEFAULT;
 	if (gfarm_ctxp->gfmd_reconnection_timeout == GFARM_CONFIG_MISC_DEFAULT)
 		gfarm_ctxp->gfmd_reconnection_timeout =
 		    GFARM_GFMD_RECONNECTION_TIMEOUT_DEFAULT;
@@ -3408,6 +3419,37 @@ gfarm_config_set_default_misc(void)
 	gfarm_config_set_default_metadb_server();
 }
 
+gfarm_error_t
+gfarm_sockbuf_apply_limit(int sock, int opt, int limit, const char *optname)
+{
+	int rv, save_errno, oldval;
+	socklen_t vallen = sizeof(oldval);
+
+	if (limit == GFARM_SOCKBUF_LIMIT_UNLIMITED)
+		return (GFARM_ERR_NO_ERROR);
+
+	rv = getsockopt(sock, SOL_SOCKET, opt, &oldval, &vallen);
+	if (rv == -1) {
+		save_errno = errno;
+		gflog_error_errno(GFARM_MSG_1004306, "%s: getsockopt",
+		    optname);
+		return (gfarm_errno_to_error(save_errno));
+	}
+	if (oldval <= limit)
+		return (GFARM_ERR_NO_ERROR);
+
+	rv = setsockopt(sock, SOL_SOCKET, opt, &limit, sizeof(limit));
+	if (rv != 0) {
+		save_errno = errno;
+		gflog_error_errno(GFARM_MSG_1004307,
+		    "%s: limiting to %d (old: %d)", optname, limit, oldval);
+		return (gfarm_errno_to_error(save_errno));
+	}
+	gflog_info(GFARM_MSG_1004308, "%s: limiting to %d (old: %d)",
+	    optname, limit, oldval);
+	return (GFARM_ERR_NO_ERROR);
+}
+
 void
 gfs_display_timers(void)
 {
