@@ -472,7 +472,7 @@ search_idle_network_list_add0(struct sockaddr *addr, int flags,
 }
 
 static gfarm_error_t
-serch_idle_network_list_local_host_init(void)
+search_idle_network_list_local_host_init(void)
 {
 	int count, i, j;
 	struct in_addr *self_ip;
@@ -515,7 +515,7 @@ search_idle_network_list_init(struct gfm_connection *gfm_server)
 
 	assert(staticp->search_idle_network_list == NULL);
 
-	e = serch_idle_network_list_local_host_init();
+	e = search_idle_network_list_local_host_init();
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
 
@@ -587,17 +587,21 @@ static gfarm_error_t
 search_idle_host_state_init(struct gfm_connection *gfm_server)
 {
 	gfarm_error_t e;
+	static const char diag[] = "search_idle_host_state_init";
 
 	e = gfp_conn_hash_table_init(&staticp->search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE);
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_debug(GFARM_MSG_1001427,
-		    "search_idle_host_state_init: hash_table_init: %s",
-		    gfarm_error_string(e));
+		    "%s: hash_table_init: %s", diag, gfarm_error_string(e));
 		return (e);
 	}
 
-	search_idle_network_list_init(gfm_server); /* ignore any error here */
+	/* ignore any error here */
+	e = search_idle_network_list_init(gfm_server);
+	if (e != GFARM_ERR_NO_ERROR)
+		gflog_notice(GFARM_MSG_1004312, "%s: network_list_init: %s",
+		    diag, gfarm_error_string(e));
 
 	/* when a connection error happens, make the host unavailable. */
 	gfs_client_add_hook_for_connection_error(
@@ -1020,13 +1024,13 @@ search_idle_init_state(struct search_idle_state *s,
 	 * If we don't check enough_number or desired_number here,
 	 * the behavior of search_idle() becomes undeterministic.
 	 * i.e. If there is a candidate, search_idle() returns NULL,
-	 * otherwise GFARM_ERRMSG_NO_FILESYSTEM_NODE.
+	 * otherwise GFARM_ERR_NO_FILESYSTEM_NODE.
 	 */
 	if (s->enough_number == 0 || s->desired_number == 0 ||
 	    staticp->search_idle_candidate_list == NULL) {
 		gflog_debug(GFARM_MSG_1001437,
 		    "search_idle_init_state: no answer is requested");
-		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+		return (GFARM_ERR_NO_FILESYSTEM_NODE);
 	}
 
 	syserr = gfarm_eventqueue_alloc(CONCURRENCY, &s->q);
@@ -1052,19 +1056,25 @@ search_idle_count(struct search_idle_state *s,
 	long long loadavg = h->loadavg;
 	int ncpu = h->ncpu;
 	int ok = 1;
+	char *msg1, *msg2, *msg3;
 
+	msg1 = msg2 = msg3 = "";
 	if (s->write_mode &&
 	    (h->flags & HOST_STATE_FLAG_STATFS_AVAIL) != 0 &&
 	    h->diskavail < gfarm_get_minimum_free_disk_space())
 		ok = 0; /* not enough free space */
-	if (ok)
+	if (ok) {
 		(*usable_numberp)++;
+		msg1 = "usable ";
+	}
 
 	if (ncpu <= 0) /* sanity */
 		ncpu = 1;
 	loadavg += h->scheduled * VIRTUAL_LOAD_FOR_SCHEDULED_HOST;
-	if (ok && loadavg / ncpu <= IDLE_LOAD_AVERAGE)
+	if (ok && loadavg / ncpu <= IDLE_LOAD_AVERAGE) {
 		(*idle_numberp)++;
+		msg2 = "idle ";
+	}
 
 	/*
 	 * We don't use (loadavg / h->host_info->ncpu) to count
@@ -1072,8 +1082,13 @@ search_idle_count(struct search_idle_state *s,
 	 * that there is a process which is consuming 100% of
 	 * memory or 100% of I/O bandwidth on the host.
 	 */
-	if (ok && loadavg <= SEMI_IDLE_LOAD_AVERAGE)
+	if (ok && loadavg <= SEMI_IDLE_LOAD_AVERAGE) {
 		(*semi_idle_numberp)++;
+		msg3 = "semi-idle ";
+	}
+	gflog_debug(GFARM_MSG_1004313, "%s: %s%s%s"
+	    "(avail %llu load %lld ncpu %d)", h->return_value, msg1, msg2, msg3,
+	    (unsigned long long)h->diskavail, loadavg, ncpu);
 }
 
 static void
@@ -1708,21 +1723,26 @@ search_idle(struct gfm_connection *gfm_server,
 	/*
 	 * 1. search local hosts
 	 */
-	if (staticp->search_idle_local_host != NULL)
+	if (staticp->search_idle_local_host != NULL) {
+		gflog_debug(GFARM_MSG_1004314, "== search local host ==");
 		search_idle_in_networks(&s,
 			staticp->search_idle_local_host_count,
 			staticp->search_idle_local_host);
+	}
 	/*
 	 * 2. search hosts on the local network
 	 *   (i.e. the same network with this client host).
 	 */
 	if (((write_mode && !gfarm_schedule_write_local_priority())
 	     || !search_idle_is_satisfied(&s))
-	    && staticp->search_idle_local_net != NULL)
+	    && staticp->search_idle_local_net != NULL) {
+		gflog_debug(GFARM_MSG_1004315, "== search local network ==");
 		search_idle_in_networks(&s, 1, &staticp->search_idle_local_net);
+	}
 	gfs_profile(gfarm_gettimerval(&t3));
 
 	if (!search_idle_is_satisfied(&s)) {
+		gflog_debug(GFARM_MSG_1004316, "== search all networks ==");
 		search_idle_examine_rtt_of_all_networks(&s);
 		e = search_idle_by_rtt_order(&s);
 	}
@@ -1745,7 +1765,7 @@ search_idle(struct gfm_connection *gfm_server,
 	if (s.usable_hosts_number == 0) {
 		gflog_debug(GFARM_MSG_1001450,
 		    "search_idle: no filesystem node");
-		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+		return (GFARM_ERR_NO_FILESYSTEM_NODE);
 	}
 
 	assert(s.available_hosts_number >= s.usable_hosts_number);
@@ -1795,6 +1815,8 @@ search_idle(struct gfm_connection *gfm_server,
 	for (i = 0; i < n && i < s.desired_number; i++) {
 		ohosts[i] = results[i]->return_value;
 		oports[i] = results[i]->port;
+		gflog_debug(GFARM_MSG_1004317, "[%d] %s:%d",
+		    i, ohosts[i], oports[i]);
 	}
 	*nohostsp = i;
 	free(results);
@@ -1827,7 +1849,7 @@ search_idle_cyclic(struct gfm_connection *gfm_server,
 	if (e != GFARM_ERR_NO_ERROR)
 		return (e);
 	if (nfound == 0)
-		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+		return (GFARM_ERR_NO_FILESYSTEM_NODE);
 	if (nohosts > nfound)
 		hosts_expand_cyclic(nfound, ohosts, oports,
 		    nohosts - nfound, &ohosts[nfound], &oports[nfound]);
@@ -1931,7 +1953,7 @@ gfarm_schedule_select_host(struct gfm_connection *gfm_server,
 	SCHED_MUTEX_LOCK(staticp)
 	e = select_hosts(gfm_server, 1, write_mode, target_domain,
 		nhosts, infos, &n, &host, &port);
-	if (target_domain != NULL && e == GFARM_ERRMSG_NO_FILESYSTEM_NODE)
+	if (target_domain != NULL && e == GFARM_ERR_NO_FILESYSTEM_NODE)
 		e = select_hosts(gfm_server, 1, write_mode, NULL,
 			nhosts, infos, &n, &host, &port);
 	SCHED_MUTEX_UNLOCK(staticp)
@@ -1940,7 +1962,7 @@ gfarm_schedule_select_host(struct gfm_connection *gfm_server,
 	if (n == 0) { /* although this shouldn't happen */
 		gflog_debug(GFARM_MSG_1001456,
 		    "gfarm_schedule_select_host: no filesystem node");
-		return (GFARM_ERRMSG_NO_FILESYSTEM_NODE);
+		return (GFARM_ERR_NO_FILESYSTEM_NODE);
 	}
 	host = strdup(host);
 	if (host == NULL) {
@@ -1987,7 +2009,7 @@ select_hosts_by_path_rpc(struct gfm_connection **gfm_serverp, void *closure)
 		gflog_debug(GFARM_MSG_1003992,
 		    "select_hosts: %s",
 		    gfarm_error_string(e));
-	if (si->target_domain != NULL && e == GFARM_ERRMSG_NO_FILESYSTEM_NODE)
+	if (si->target_domain != NULL && e == GFARM_ERR_NO_FILESYSTEM_NODE)
 		e = select_hosts(*gfm_serverp, si->acyclic, si->write_mode,
 			NULL, si->ninfos, si->infos, si->nohostsp, si->ohosts,
 			si->oports);
@@ -2087,6 +2109,9 @@ gfarm_schedule_host_used(const char *hostname, int port, const char *username)
 	struct gfarm_hash_entry *entry;
 	struct search_idle_host_state *h;
 
+	if (staticp->search_idle_hosts_state == NULL)
+		return (HOST_STATE_SCHEDULED_AGE_NOT_FOUND);
+
 	e = gfp_conn_hash_lookup(&staticp->search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE, hostname, port, username, &entry);
 	if (e != GFARM_ERR_NO_ERROR)
@@ -2104,10 +2129,14 @@ gfarm_schedule_host_unused(const char *hostname, int port, const char *username,
 	gfarm_error_t e;
 	struct gfarm_hash_entry *entry;
 	struct search_idle_host_state *h;
+	static const char diag[] = "search_host_unused";
 
 	if (scheduled_age == HOST_STATE_SCHEDULED_AGE_NOT_FOUND)
 		return;
-
+	if (staticp->search_idle_hosts_state == NULL) {
+		gflog_notice(GFARM_MSG_1004318, "%s: internal error", diag);
+		return;
+	}
 	e = gfp_conn_hash_lookup(&staticp->search_idle_hosts_state,
 	    HOSTS_HASHTAB_SIZE, hostname, port, username, &entry);
 	if (e != GFARM_ERR_NO_ERROR)
