@@ -34,12 +34,6 @@
 
 /* #define SS_DEBUG */
 
-/*
- * Initial credential and its name.
- */
-static gss_cred_id_t initiatorInitialCred = GSS_C_NO_CREDENTIAL;
-static gss_cred_id_t acceptorInitialCred = GSS_C_NO_CREDENTIAL;
-
 static int initiatorInitialized = 0;
 static int acceptorInitialized = 0;
 
@@ -186,7 +180,7 @@ dumpSsOpt(gfarmSecSessionOption *ssOptPtr)
 }
 #endif /* SS_DEBUG */
 
-
+/* thread safe */
 static int
 secSessionReadConfigFile(char *configFile, gfarmSecSessionOption *ssOptPtr)
 {
@@ -400,7 +394,7 @@ negotiateConfigParam(int fd, gss_ctx_id_t sCtx, int which,
 	    int iConf, iConfF;
 	   
 	    if (gfarmReadInt32(fd, param, NUM_NEGO_PARAM,
-			       GFARM_GSS_TIMEOUT_INFINITE) != NUM_NEGO_PARAM) {
+		GFARM_GSS_AUTH_TIMEOUT_MSEC) != NUM_NEGO_PARAM) {
 		gsiErrNo = errno;
 		gflog_auth_info(GFARM_MSG_1003849,
 		    "gfarmSecSession:negotiateConfigParam(): "
@@ -438,12 +432,12 @@ negotiateConfigParam(int fd, gss_ctx_id_t sCtx, int which,
 		/*
 		 * Use the acceptor's.
 		 */
-		retMaxT = iMax;
+		retMaxT = canPtr->maxTransSizeReq;
 	    } else if (iMaxF == 1) {
 		/*
 		 * Use the initiator's.
 		 */
-		retMaxT = canPtr->maxTransSizeReq;
+		retMaxT = iMax;
 	    } else { 
 		/*
 		 * Both force flags are off.
@@ -504,7 +498,7 @@ negotiateConfigParam(int fd, gss_ctx_id_t sCtx, int which,
 	    }
 
 	    if (gfarmReadInt32(fd, param, NUM_NEGO_PARAM,
-			       GFARM_GSS_TIMEOUT_INFINITE) != NUM_NEGO_PARAM) {
+		GFARM_GSS_AUTH_TIMEOUT_MSEC) != NUM_NEGO_PARAM) {
 		gsiErrNo = errno;
 		gflog_auth_error(GFARM_MSG_1000665,
 		    "gfarmSecSession:negotiateConfigParam(): "
@@ -604,46 +598,30 @@ destroySecSession(gfarmSecSession *ssPtr)
     if (ssPtr != NULL) {
 	OM_uint32 minStat;
 
-	if (ssPtr->peerName != NULL) {
-	    (void)free(ssPtr->peerName);
-	}
-
+	free(ssPtr->peerName);
 	switch (ssPtr->iOa) {
-	    case GFARM_SS_INITIATOR: {
-		if (ssPtr->iOaInfo.initiator.acceptorName != GSS_C_NO_NAME) {
+	    case GFARM_SS_INITIATOR:
+		if (ssPtr->iOaInfo.initiator.acceptorName != GSS_C_NO_NAME)
 		    gfarmGssDeleteName(&ssPtr->iOaInfo.initiator.acceptorName,
 				       NULL, NULL);
-		}
 		break;
-	    }
-	    case GFARM_SS_ACCEPTOR: {
-		if (ssPtr->iOaInfo.acceptor.initiatorName != GSS_C_NO_NAME) {
+	    case GFARM_SS_ACCEPTOR:
+		if (ssPtr->iOaInfo.acceptor.initiatorName != GSS_C_NO_NAME)
 		    gfarmGssDeleteName(&ssPtr->iOaInfo.acceptor.initiatorName,
 				       NULL, NULL);
-		}
-		if (ssPtr->iOaInfo.acceptor.deleCred != GSS_C_NO_CREDENTIAL) {
+		if (ssPtr->iOaInfo.acceptor.deleCred != GSS_C_NO_CREDENTIAL)
 		    (void)gss_release_cred(&minStat,
-					   &(ssPtr->iOaInfo.acceptor.deleCred));
-		}
+				&(ssPtr->iOaInfo.acceptor.deleCred));
 		if (ssPtr->iOaInfo.acceptor.mappedUser != NULL) {
 		    /*
 		     * ssPtr->iOaInfo.acceptor.mappedUser may be NULL,
 		     * if gfarmSecSessionAccept() aborts during initialization.
 		     */
-		    ssPtr->iOaInfo.acceptor.mappedUser->sesRefCount--;
-		    if (ssPtr->iOaInfo.acceptor.mappedUser->sesRefCount < 0) {
-			ssPtr->iOaInfo.acceptor.mappedUser->sesRefCount = 0;
-		    }
-		    if (ssPtr->iOaInfo.acceptor.mappedUser->sesRefCount == 0 &&
-			ssPtr->iOaInfo.acceptor.mappedUser->orphaned == 1) {
-			gfarmAuthDestroyUserEntry(ssPtr->iOaInfo.acceptor.mappedUser);
-		    }
+		    gfarmAuthFreeUserEntry(ssPtr->iOaInfo.acceptor.mappedUser);
 		}
 		break;
-	    }
-	    default: {
+	    default:
 		break;
-	    }
 	}
 
 	if (ssPtr->sCtx != GSS_C_NO_CONTEXT) {
@@ -691,40 +669,13 @@ gfarmSecSessionInitializeAcceptor(char *configFile, char *usermapFile,
 
     gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (acceptorInitialized == 0) {
-	char confFile[PATH_MAX];
-
-	/*
-	 * Get a credential.
-	 */
-	if (acceptorInitialCred == GSS_C_NO_CREDENTIAL) {
-	    if (gfarmGssAcquireCredential(&acceptorInitialCred,
-					  GSS_C_NO_NAME, GSS_C_ACCEPT,
-					  &majStat, &minStat, NULL) < 0) {
-		/*
-		 * This initial credential is just a default credential,
-		 * which may be used by gfarmSecSessionAccept() when
-		 * GSS_C_NO_CREDENTIAL is specified.
-		 */
-	    }
-	}
-
+	char *confFile = NULL;
 	/*
 	 * Read config file.
 	 */
 	if (configFile == NULL || configFile[0] == '\0') {
-	    char *confDir = gfarmGetEtcDir();
-	    if (confDir == NULL) {
-		gflog_auth_error(GFARM_MSG_1000666,
-		    "gfarmSecSessionInitializeAcceptor(): "
-				 "cannot access configuration directory");
-		goto SkipReadConfFile;
-	    }
-#ifdef HAVE_SNPRINTF
-	    snprintf(confFile, sizeof confFile, "%s/%s", confDir, GFARM_DEFAULT_ACCEPTOR_CONFIG_FILE);
-#else
-	    sprintf(confFile, "%s/%s", confDir, GFARM_DEFAULT_ACCEPTOR_CONFIG_FILE);
-#endif
-	    (void)free(confDir);
+	    confFile = gfarmGetDefaultConfigFile(
+		GFARM_DEFAULT_ACCEPTOR_CONFIG_FILE);
 	    configFile = confFile;
 	}
 	if (secSessionReadConfigFile(configFile, &acceptorSsOpt) < 0) {
@@ -736,7 +687,6 @@ gfarmSecSessionInitializeAcceptor(char *configFile, char *usermapFile,
 	    goto Done;
 	}
 
-    SkipReadConfFile:
 	/*
 	 * Authorization init.
 	 */
@@ -750,13 +700,8 @@ gfarmSecSessionInitializeAcceptor(char *configFile, char *usermapFile,
 	}
 
 	Done:
-	if (ret == -1) {
-	    if (acceptorInitialCred != GSS_C_NO_CREDENTIAL) {
-		OM_uint32 minStat;
-		(void)gss_release_cred(&minStat, &acceptorInitialCred);
-		acceptorInitialCred = GSS_C_NO_CREDENTIAL;
-	    }
-	} else {
+	free(confFile);
+	if (ret == 1) {
 	    acceptorInitialized = 1;
 	}
     }
@@ -771,20 +716,6 @@ gfarmSecSessionInitializeAcceptor(char *configFile, char *usermapFile,
     return ret;
 }
 
-int
-gfarmSecSessionGetInitiatorInitialCredential(gss_cred_id_t *credPtr)
-{
-    int initiator_Initialized;
-    static const char diag[] = "gfarmSecSessionGetInitiatorInitialCredential()";
-
-    gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
-    *credPtr = initiatorInitialCred;
-    initiator_Initialized = initiatorInitialized ? 1 : -1;
-    gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
-
-    return initiator_Initialized;
-}
-
 int
 gfarmSecSessionInitializeInitiator(char *configFile, char *usermapFile,
     OM_uint32 *majStatPtr, OM_uint32 *minStatPtr)
@@ -796,40 +727,13 @@ gfarmSecSessionInitializeInitiator(char *configFile, char *usermapFile,
 
     gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     if (initiatorInitialized == 0) {
-	char confFile[PATH_MAX];
-
-	/*
-	 * Get a credential.
-	 */
-	if (initiatorInitialCred == GSS_C_NO_CREDENTIAL) {
-	    if (gfarmGssAcquireCredential(&initiatorInitialCred,
-					  GSS_C_NO_NAME, GSS_C_INITIATE,
-					  &majStat, &minStat, NULL) < 0) {
-		/*
-		 * This initial credential is just a default credential,
-		 * which may be used by gfarmSecSessionInitiate() when
-		 * GSS_C_NO_CREDENTIAL is specified.
-		 */
-	    }
-	}
-
+	char *confFile = NULL;
 	/*
 	 * Read config file.
 	 */
 	if (configFile == NULL || configFile[0] == '\0') {
-	    char *confDir = gfarmGetEtcDir();
-	    if (confDir == NULL) {
-		gflog_auth_error(GFARM_MSG_1000667,
-		    "gfarmSecSessionInitializeInitiator(): "
-				 "cannot access configuration directory");
-		goto SkipReadConfFile;
-	    }
-#ifdef HAVE_SNPRINTF
-	    snprintf(confFile, sizeof confFile, "%s/%s", confDir, GFARM_DEFAULT_INITIATOR_CONFIG_FILE);
-#else
-	    sprintf(confFile, "%s/%s", confDir, GFARM_DEFAULT_INITIATOR_CONFIG_FILE);
-#endif
-	    (void)free(confDir);
+	    confFile = gfarmGetDefaultConfigFile(
+		GFARM_DEFAULT_INITIATOR_CONFIG_FILE);
 	    configFile = confFile;
 	}
 	if (secSessionReadConfigFile(configFile, &initiatorSsOpt) < 0) {
@@ -841,7 +745,6 @@ gfarmSecSessionInitializeInitiator(char *configFile, char *usermapFile,
 	    goto Done;
 	}
 
-    SkipReadConfFile:
 #if GFARM_FAKE_GSS_C_NT_USER_NAME_FOR_GLOBUS
 	/*
 	 * If GFARM_FAKE_GSS_C_NT_USER_NAME_FOR_GLOBUS is true,
@@ -857,13 +760,8 @@ gfarmSecSessionInitializeInitiator(char *configFile, char *usermapFile,
 #endif /* GFARM_FAKE_GSS_C_NT_USER_NAME_FOR_GLOBUS */
 
 	Done:
-	if (ret == -1) {
-	    if (initiatorInitialCred != GSS_C_NO_CREDENTIAL) {
-		OM_uint32 minStat;
-		(void)gss_release_cred(&minStat, &initiatorInitialCred);
-		initiatorInitialCred = GSS_C_NO_CREDENTIAL;
-	    }
-	} else {
+	free(confFile);
+	if (ret == 1) {
 	    initiatorInitialized = 1;
 	}
     }
@@ -875,7 +773,7 @@ gfarmSecSessionInitializeInitiator(char *configFile, char *usermapFile,
     if (minStatPtr != NULL) {
 	*minStatPtr = minStat;
     }
-    return ret;
+    return (ret);
 }
 
 
@@ -905,45 +803,14 @@ gfarmSecSessionInitializeBoth(char *iConfigFile, char *aConfigFile,
     gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (initiatorInitialized == 0 && acceptorInitialized == 0) {
-	char confFile[PATH_MAX];
-	char *confDir = NULL;
-
-	if (acceptorInitialCred == GSS_C_NO_CREDENTIAL) {
-	    if (gfarmGssAcquireCredential(&acceptorInitialCred,
-					  GSS_C_NO_NAME, GSS_C_BOTH,
-					  &majStat, &minStat, NULL) < 0) {
-		/*
-		 * This initial credential is just a default credential,
-		 * which may be used by gfarmSecSessionAccept() when
-		 * GSS_C_NO_CREDENTIAL is specified.
-		 */
-	    }
-	    initiatorInitialCred = acceptorInitialCred;
-	}
-
-	/*
-	 * Read config file.
-	 */
-	if ((aConfigFile == NULL || aConfigFile[0] == '\0') ||
-	    (iConfigFile == NULL || iConfigFile[0] == '\0')) {
-	    confDir = gfarmGetEtcDir();
-	    if (confDir == NULL) {
-		gflog_auth_error(GFARM_MSG_1000668,
-		    "gfarmSecSessionInitializeBoth(): "
-				 "cannot access configuration directory");
-		goto SkipReadConfFile;
-	    }
-	}
+	char *confFile = NULL;
 
 	/*
 	 * Acceptor's configuration
 	 */
 	if (aConfigFile == NULL || aConfigFile[0] == '\0') {
-#ifdef HAVE_SNPRINTF
-	    snprintf(confFile, sizeof confFile, "%s/%s", confDir, GFARM_DEFAULT_ACCEPTOR_CONFIG_FILE);
-#else
-	    sprintf(confFile, "%s/%s", confDir, GFARM_DEFAULT_ACCEPTOR_CONFIG_FILE);
-#endif
+	    confFile = gfarmGetDefaultConfigFile(
+		GFARM_DEFAULT_ACCEPTOR_CONFIG_FILE);
 	    aConfigFile = confFile;
 	}
 	if (secSessionReadConfigFile(aConfigFile, &acceptorSsOpt) < 0) {
@@ -954,16 +821,13 @@ gfarmSecSessionInitializeBoth(char *iConfigFile, char *aConfigFile,
 		"failed to read acceptor's config file (%s)", aConfigFile);
 	    goto Done;
 	}
-
+	free(confFile);
 	/*
 	 * Initiator's configuration
 	 */
 	if (iConfigFile == NULL || iConfigFile[0] == '\0') {
-#ifdef HAVE_SNPRINTF
-	    snprintf(confFile, sizeof confFile, "%s/%s", confDir, GFARM_DEFAULT_INITIATOR_CONFIG_FILE);
-#else
-	    sprintf(confFile, "%s/%s", confDir, GFARM_DEFAULT_INITIATOR_CONFIG_FILE);
-#endif
+	    confFile = gfarmGetDefaultConfigFile(
+		GFARM_DEFAULT_INITIATOR_CONFIG_FILE);
 	    iConfigFile = confFile;
 	}
 	if (secSessionReadConfigFile(iConfigFile, &initiatorSsOpt) < 0) {
@@ -975,7 +839,6 @@ gfarmSecSessionInitializeBoth(char *iConfigFile, char *aConfigFile,
 	    goto Done;
 	}
 
-    SkipReadConfFile:
 	/*
 	 * Authorization init.
 	 */
@@ -989,16 +852,8 @@ gfarmSecSessionInitializeBoth(char *iConfigFile, char *aConfigFile,
 	}
 
 	Done:
-	if (confDir != NULL) {
-	    (void)free(confDir);
-	}
-	if (ret == -1) {
-	    if (acceptorInitialCred != GSS_C_NO_CREDENTIAL) {
-		gfarmGssDeleteCredential(&acceptorInitialCred, NULL, NULL);
-		acceptorInitialCred = GSS_C_NO_CREDENTIAL;
-	    }
-	    initiatorInitialCred = GSS_C_NO_CREDENTIAL;
-	} else {
+	free(confFile);
+	if (ret == 1) {
 	    initiatorInitialized = 1;
 	    acceptorInitialized = 1;
 	}
@@ -1023,10 +878,6 @@ gfarmSecSessionFinalizeInitiator(void)
 
     gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     if (initiatorInitialized == 1) {
-	if (initiatorInitialCred != GSS_C_NO_CREDENTIAL) {
-	    gfarmGssDeleteCredential(&initiatorInitialCred, NULL, NULL);
-	    initiatorInitialCred = GSS_C_NO_CREDENTIAL;
-	}
 	initiatorInitialized = 0;
     }
     gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
@@ -1040,10 +891,6 @@ gfarmSecSessionFinalizeAcceptor(void)
 
     gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (acceptorInitialized == 1) {
-	if (acceptorInitialCred != GSS_C_NO_CREDENTIAL) {
-	    gfarmGssDeleteCredential(&acceptorInitialCred, NULL, NULL);
-	    acceptorInitialCred = GSS_C_NO_CREDENTIAL;
-	}
 	gfarmAuthFinalize();
 	acceptorInitialized = 0;
     }
@@ -1059,15 +906,6 @@ gfarmSecSessionFinalizeBoth(void)
     gfarm_mutex_lock(&initiator_mutex, diag, initiatorDiag);
     gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
     if (initiatorInitialized == 1 && acceptorInitialized == 1) {
-	if (acceptorInitialCred != GSS_C_NO_CREDENTIAL) {
-	    gfarmGssDeleteCredential(&acceptorInitialCred, NULL, NULL);
-	    acceptorInitialCred = GSS_C_NO_CREDENTIAL;
-	}
-	/*
-	 * do not delete the initial initiator credential, since it is
-	 * not allocated.
-	 */
-	initiatorInitialCred = GSS_C_NO_CREDENTIAL;
 	gfarmAuthFinalize();
 	acceptorInitialized = 0;
 	initiatorInitialized = 0;
@@ -1076,32 +914,6 @@ gfarmSecSessionFinalizeBoth(void)
     gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 }
 
-/*
- * This function is intended to check the validity of the initial
- * acceptor certificate.  Unfortunately, it cannot check the
- * expiration of CA and CRL.
- */
-int
-gfarmSecSessionAcceptorCredIsValid(OM_uint32 *majStatPtr, OM_uint32 *minStatPtr)
-{
-    OM_uint32 majStat;
-    static const char diag[] = "gfarmSecSessionAcceptorCredIsValid()";
-
-    gfarm_mutex_lock(&acceptor_mutex, diag, acceptorDiag);
-
-    /* gss_inquire_cred() may call gss_acquire_cred() internally */
-    gfarm_privilege_lock(diag);
-    majStat = gss_inquire_cred(
-	minStatPtr, acceptorInitialCred, NULL, NULL, NULL, NULL);
-    gfarm_privilege_unlock(diag);
-
-    gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
-    if (majStatPtr != NULL)
-	*majStatPtr = majStat;
-
-    return (majStat == GSS_S_COMPLETE ? 1 : 0);
-}
-
 gfarmSecSession *
 gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
     gfarmSecSessionOption *ssOptPtr, int *gsiErrNoPtr, OM_uint32 *majStatPtr,
@@ -1166,13 +978,6 @@ gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
 		"gfarmGetPeernameOfSocket() failed: %s", strerror(gsiErrNo));
 	goto Fail;
     }
-
-    /*
-     * Check the credential.
-     */
-    if (cred == GSS_C_NO_CREDENTIAL) {
-	cred = acceptorInitialCred;
-    }
     gfarm_mutex_unlock(&acceptor_mutex, diag, acceptorDiag);
 
     /*
@@ -1206,19 +1011,17 @@ gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
 	gflog_auth_notice(GFARM_MSG_1000672,
 	    "%s: not registered in mapfile", initiatorDistName);
 	majStat = GSS_S_UNAUTHORIZED;
-	/* Send NACK. */
-	acknack = GFARM_SS_AUTH_NACK;
-	(void)gfarmWriteInt32(fd, &acknack, 1);
-	goto Fail;
+	goto SendNack;
     } else {
 	int type = gfarmAuthGetAuthEntryType(entry);
+
 	if (type == GFARM_AUTH_USER) {
 	    /* Send ACK. */
 	    acknack = GFARM_SS_AUTH_ACK;
 	    (void)gfarmWriteInt32(fd, &acknack, 1);
 	} else if (type == GFARM_AUTH_HOST) {
 	    /* check peer name is actually allowed */
-	    if (strcmp(peerName, entry->authData.hostAuth.FQDN) == 0) {
+	    if (strcmp(peerName, gfarmAuthGetFQDN(entry)) == 0) {
 		/* Send ACK. */
 		acknack = GFARM_SS_AUTH_ACK;
 		(void)gfarmWriteInt32(fd, &acknack, 1);
@@ -1226,12 +1029,10 @@ gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
 		gflog_auth_error(GFARM_MSG_1000673,
 		    "%s: hostname doesn't match: "
 		    "peer %s, expected %s", initiatorDistName, peerName,
-		    entry->authData.hostAuth.FQDN);
+		    gfarmAuthGetFQDN(entry));
 		majStat = GSS_S_UNAUTHORIZED;
-		/* Send NACK. */
-		acknack = GFARM_SS_AUTH_NACK;
-		(void)gfarmWriteInt32(fd, &acknack, 1);
-		goto Fail;
+		gfarmAuthFreeUserEntry(entry);
+		goto SendNack;
 	    }
 	}
     }
@@ -1246,6 +1047,7 @@ gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
 	gflog_debug(GFARM_MSG_1003855,
 		"negotiateConfigParam() failed (%s)(%u)(%u)",
 		strerror(gsiErrNo), majStat, minStat);
+	gfarmAuthFreeUserEntry(entry);
 	goto Fail;
     }
 #if 0
@@ -1263,7 +1065,6 @@ gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
     ret->sCtx = sCtx;
     ret->iOa = GFARM_SS_ACCEPTOR;
     ret->iOaInfo.acceptor.mappedUser = entry;
-    ret->iOaInfo.acceptor.mappedUser->sesRefCount++;
     ret->iOaInfo.acceptor.initiatorName = initiatorName;
     ret->iOaInfo.acceptor.deleCred = deleCred;
     ret->qOp = qOp;
@@ -1272,7 +1073,11 @@ gfarmSecSessionAccept(int fd, gss_cred_id_t cred,
     ret->gssLastStat = majStat;
     goto Done;
 
-    Fail:
+SendNack:
+    /* Send NACK. */
+    acknack = GFARM_SS_AUTH_NACK;
+    gfarmWriteInt32(fd, &acknack, 1);
+Fail:
     if (ret != NULL) {
 	destroySecSession(ret);
 	ret = NULL;
@@ -1370,13 +1175,6 @@ secSessionInitiate(int fd, const gss_name_t acceptorName,
 		"gfarmGetPeernameOfSocket() failed: %s", strerror(gsiErrNo));
 	goto Fail;
     }
-
-    /*
-     * Check the credential.
-     */
-    if (cred == GSS_C_NO_CREDENTIAL) {
-	cred = initiatorInitialCred;
-    }
     gfarm_mutex_unlock(&initiator_mutex, diag, initiatorDiag);
 
     /*
@@ -1405,7 +1203,7 @@ secSessionInitiate(int fd, const gss_name_t acceptorName,
     /*
      * Phase 2: Receive authorization acknowledgement.
      */
-    if (gfarmReadInt32(fd, &acknack, 1, GFARM_GSS_TIMEOUT_INFINITE) != 1) {
+    if (gfarmReadInt32(fd, &acknack, 1, GFARM_GSS_AUTH_TIMEOUT_MSEC) != 1) {
 	gsiErrNo = errno;
 	gflog_auth_error(GFARM_MSG_1003860,
 	    "%s: acceptor does not answer authentication result: %s",
@@ -1557,15 +1355,16 @@ gfarmSecSessionGetInitiatorInfo(gfarmSecSession *ssPtr)
     return ssPtr->iOaInfo.acceptor.mappedUser;
 }
 
-
 int
 gfarmSecSessionDedicate(gfarmSecSession *ssPtr)
 {
     int ret = -1;
     gfarmAuthEntry *aePtr = gfarmSecSessionGetInitiatorInfo(ssPtr);
+
     if (aePtr != NULL) {
 	gid_t gid = getgid();
 	uid_t uid = getuid();
+
 #if ! GFARM_FAKE_GSS_C_NT_USER_NAME_FOR_GLOBUS
 	/*
 	 * If GFARM_FAKE_GSS_C_NT_USER_NAME_FOR_GLOBUS is true,
@@ -1574,16 +1373,16 @@ gfarmSecSessionDedicate(gfarmSecSession *ssPtr)
 	 */
 	gfarmAuthMakeThisAlone(aePtr);
 #endif /* ! GFARM_FAKE_GSS_C_NT_USER_NAME_FOR_GLOBUS */
-	if (uid == 0 && aePtr->authType == GFARM_AUTH_USER) {
-	    if (aePtr->authData.userAuth.gid != gid) {
-		if ((ret = setgid(aePtr->authData.userAuth.gid)) < 0) {
+	if (uid == 0 && gfarmAuthGetAuthEntryType(aePtr) == GFARM_AUTH_USER) {
+	    if (gfarmAuthGetGid(aePtr) != gid) {
+		if ((ret = setgid(gfarmAuthGetGid(aePtr))) < 0) {
 		    ssPtr->gssLastStat = GSS_S_FAILURE;
 		    gflog_debug(GFARM_MSG_1000833, "setgid() failed");
 		    goto Done;
 		}
 	    }
-	    if (aePtr->authData.userAuth.uid != uid) {
-		if ((ret = setuid(aePtr->authData.userAuth.uid)) < 0) {
+	    if (gfarmAuthGetUid(aePtr) != uid) {
+		if ((ret = setuid(gfarmAuthGetUid(aePtr))) < 0) {
 		    ssPtr->gssLastStat = GSS_S_FAILURE;
 		    gflog_debug(GFARM_MSG_1000834, "setuid() failed");
 		    goto Done;
@@ -1600,7 +1399,6 @@ gfarmSecSessionDedicate(gfarmSecSession *ssPtr)
     return ret;
 }
 
-
 int
 gfarmSecSessionSendInt8(gfarmSecSession *ssPtr, gfarm_int8_t *buf, int n)
 {
@@ -1895,7 +1693,7 @@ negotiateConfigParamInitiatorReceive(int events, int fd, void *closure,
     } else {
 	assert(events == GFARM_EVENT_READ);
 	if (gfarmReadInt32(fd, param, NUM_NEGO_PARAM,
-			   GFARM_GSS_TIMEOUT_INFINITE) != NUM_NEGO_PARAM) {
+	    GFARM_GSS_AUTH_TIMEOUT_MSEC) != NUM_NEGO_PARAM) {
 	    gflog_auth_error(GFARM_MSG_1000680, "gfarmSecSession: "
 			     "negotiateConfigParamInitiatorReceive(): "
 			     "negotiation failure with the acceptor");
@@ -2155,7 +1953,7 @@ secSessionInitiateReceiveAuthorizationAck(int events, int fd, void *closure,
 	/*
 	 * Phase 2: Receive authorization acknowledgement.
 	 */
-	if (gfarmReadInt32(fd, &acknack, 1, GFARM_GSS_TIMEOUT_INFINITE) != 1) {
+	if (gfarmReadInt32(fd, &acknack, 1, GFARM_GSS_AUTH_TIMEOUT_MSEC) != 1) {
 	    gflog_auth_error(GFARM_MSG_1000689,
 	        "%s: acceptor does not answer authentication result",
 		state->acceptorDistName);
@@ -2282,13 +2080,6 @@ secSessionInitiateRequest(struct gfarm_eventqueue *q, int fd,
 	 */
 	(void)gfarmGetPeernameOfSocket(fd, &state->rPort, &state->peerName);
 
-	/*
-	 * Check the credential.
-	 */
-	if (cred == GSS_C_NO_CREDENTIAL) {
-	    state->cred = initiatorInitialCred;
-	}
-
 	state->readable = gfarm_fd_event_alloc(
 	    GFARM_EVENT_READ|GFARM_EVENT_TIMEOUT, fd,
 	    secSessionInitiateReceiveAuthorizationAck, state);
