@@ -13,7 +13,7 @@ cleanup() {
     gfhost -d ${host} > /dev/null 2>&1
 }
 
-trap 'celanup; exit $exit_trap' $trap_sigs
+trap 'cleanup; exit $exit_trap' $trap_sigs
 
 gfhost -c -a dummy -p 12345 ${host} > /dev/null 2>&1
 if test $? -ne 0; then
