@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 9779 2015-06-30 06:31:35Z takuya-i $
+ * $Id: gfmd.c 9872 2015-08-25 06:29:05Z takuya-i $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -490,6 +490,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 		e = gfm_server_replica_create_file_in_lost_found(peer,
 		    from_client, skip);
 		break;
+	case GFM_PROTO_REPLICA_OPEN_STATUS:
+		e = gfm_server_replica_open_status(peer, from_client, skip);
+		break;
 	case GFM_PROTO_PROCESS_ALLOC:
 		e = gfm_server_process_alloc(peer, from_client, skip);
 		break;
@@ -1215,8 +1218,6 @@ transform_to_master(void)
 	/* this must be after db_journal_wait_for_apply_thread() */
 	dead_file_copy_init_load();
 
-	quota_check();
-
 	giant_unlock();
 
 	gfarm_cond_signal(&transform_cond, diag, TRANSFORM_COND_DIAG);
@@ -1794,7 +1795,6 @@ main(int argc, char **argv)
 		inode_remove_orphan(); /* should be before
 					  inode_check_and_repair() */
 		inode_check_and_repair();
-		quota_check();
 	}
 	inode_free_orphan();
 	gflog_info(GFARM_MSG_1004204, "end bootstrap");
@@ -1814,6 +1814,7 @@ main(int argc, char **argv)
 	/* master */
 
 	failover_notify();
+	quota_check_init();
 	replica_check_init();
 	accepting_loop(sock);
 
