@@ -533,6 +533,12 @@ gfmdc_wait_journal_syncsend(struct gfmdc_journal_send_closure *c)
 	if ((r = c->end) == 0)
 		c->end = 1;
 	gfarm_mutex_unlock(&c->send_mutex, diag, SEND_MUTEX_DIAG);
+	if (in_time == 0)
+		gflog_notice(GFARM_MSG_1004270,
+		    "journal send to %s: timeout due to no response "
+		    "within %d seconds "
+		    "(synchronous_journal_timeout)", mdhost_get_name(c->host),
+		     gfarm_get_journal_sync_slave_timeout());
 	return (r);
 }
 
@@ -579,7 +585,7 @@ gfmdc_client_journal_send(struct peer *peer,
 	    GFM_PROTO_JOURNAL_SEND, "llb", from_sn, to_sn,
 	    (size_t)data_len, data)) != GFARM_ERR_NO_ERROR) {
 		gflog_error(GFARM_MSG_1002978,
-		    "%s : %s", mdhost_get_name(mh), gfarm_error_string(e));
+		    "%s: %s", mdhost_get_name(mh), gfarm_error_string(e));
 		free(data);
 		c->data = NULL;
 		return (e);
@@ -651,6 +657,7 @@ gfmdc_server_journal_ready_to_recv(struct mdhost *mh, struct peer *peer,
 	int inited = 0;
 	struct gfmdc_peer_record *gfmdc_peer = peer_get_gfmdc_record(peer);
 	struct journal_file_reader *reader;
+	const char *host = mdhost_get_name(mh);
 	static const char diag[] = "GFM_PROTO_JOURNAL_READY_TO_RECV";
 
 	if ((e = gfmdc_server_get_request(peer, size, diag, "l", &seqnum))
@@ -660,10 +667,10 @@ gfmdc_server_journal_ready_to_recv(struct mdhost *mh, struct peer *peer,
 #ifdef DEBUG_JOURNAL
 		gflog_debug(GFARM_MSG_1002980,
 		    "%s : last_fetch_seqnum=%llu",
-		    mdhost_get_name(mh), (unsigned long long)seqnum);
+		    host, (unsigned long long)seqnum);
 #endif
 		reader = gfmdc_peer_get_journal_file_reader(gfmdc_peer);
-		if ((e = db_journal_reader_reopen_if_needed(&reader,
+		if ((e = db_journal_reader_reopen_if_needed(host, &reader,
 		    gfmdc_peer_get_last_fetch_seqnum(gfmdc_peer), &inited))
 		    == GFARM_ERR_NO_ERROR) {
 			if (seqnum == db_journal_get_current_seqnum())
@@ -674,8 +681,7 @@ gfmdc_server_journal_ready_to_recv(struct mdhost *mh, struct peer *peer,
 			mdhost_set_seqnum_state_by_error(mh, e);
 			gflog_error(GFARM_MSG_1002981,
 			    "gfmd_channel(%s) : %s",
-			    mdhost_get_name(mh),
-			    gfarm_error_string(e));
+			    host, gfarm_error_string(e));
 		}
 		/*
 		 * if reader is already expired,
@@ -726,7 +732,7 @@ gfmdc_client_journal_ready_to_recv_result(void *p, void *arg, size_t size)
 
 	if ((e = gfmdc_client_recv_result(peer, mh, size, diag, ""))
 	    != GFARM_ERR_NO_ERROR)
-		gflog_error(GFARM_MSG_1002982, "%s : %s",
+		gflog_error(GFARM_MSG_1002982, "%s: %s",
 		    mdhost_get_name(mh), gfarm_error_string(e));
 	gfmdc_client_journal_ready_to_recv_notify(arg, e, diag);
 	return (e);
@@ -765,7 +771,7 @@ gfmdc_client_journal_ready_to_recv(struct mdhost *mh, struct peer *peer)
 	    GFM_PROTO_JOURNAL_READY_TO_RECV, "l", seqnum))
 	    != GFARM_ERR_NO_ERROR) {
 		gflog_error(GFARM_MSG_1002983,
-		    "%s : %s", mdhost_get_name(mh), gfarm_error_string(e));
+		    "%s: %s", mdhost_get_name(mh), gfarm_error_string(e));
 		goto end;
 	}
 
@@ -779,7 +785,7 @@ gfmdc_client_journal_ready_to_recv(struct mdhost *mh, struct peer *peer)
 	e = ji.error;
 	if (e != GFARM_ERR_NO_ERROR) {
 		gflog_error(GFARM_MSG_1003425,
-		    "%s : %s", mdhost_get_name(mh), gfarm_error_string(e));
+		    "%s: %s", mdhost_get_name(mh), gfarm_error_string(e));
 	}
 
 end:
@@ -1208,7 +1214,8 @@ gfmdc_connect_thread(void *arg)
 		if (peer != NULL) { /* already connected to the master? */
 			mdhost_put_peer(mh, peer); /* decrement refcount */
 			sleep(GFMDC_CONNECT_INTERVAL);
-		} else if ((e = gfmdc_connect()) != GFARM_ERR_NO_ERROR) {
+		} else if ((e = gfmdc_connect()) != GFARM_ERR_NO_ERROR &&
+		    !gfm_client_is_connection_error(e)) {
 			gflog_error(GFARM_MSG_1003430,
 			    "gfmd_channel : "
 			    "give up to connect to the master gfmd: %s",
@@ -1249,6 +1256,7 @@ gfmdc_journal_syncsend_thread(void *closure)
 	struct gfmdc_journal_send_closure *c =
 	    gfmdc_peer_get_journal_send_closure(gfmdc_peer);
 	struct mdhost *mh = peer_get_mdhost(peer);
+	struct journal_file_reader *reader;
 	gfarm_error_t e;
 	gfarm_uint64_t to_sn;
 	static const char diag[] = "gfmdc_journal_send_thread";
@@ -1267,13 +1275,13 @@ gfmdc_journal_syncsend_thread(void *closure)
 		if ((e = gfmdc_client_journal_syncsend(peer, c, &to_sn))
 		    != GFARM_ERR_NO_ERROR) {
 			gflog_error(GFARM_MSG_1003003,
-			    "%s : failed to send journal : %s",
+			    "%s: failed to send journal: %s",
 			    mdhost_get_name(c->host), gfarm_error_string(e));
 			break;
 		}
 		if (to_sn == 0) {
 			gflog_warning(GFARM_MSG_1003004,
-			    "%s : no journal records to send (seqnum=%lld) "
+			    "%s: no journal record to send (seqnum=%lld)"
 			    ": %s", mdhost_get_name(c->host),
 			    (unsigned long long)journal_sync_info.seqnum,
 			    gfarm_error_string(e));
@@ -1281,9 +1289,15 @@ gfmdc_journal_syncsend_thread(void *closure)
 		}
 	} while (to_sn < journal_sync_info.seqnum);
 
-	if (e != GFARM_ERR_NO_ERROR)
+	if (e != GFARM_ERR_NO_ERROR) {
+		/*
+		 * send signal to nonfull cond just in case the writer
+		 * waits for this reader thread
+		 */
+		reader = gfmdc_peer_get_journal_file_reader(gfmdc_peer);
+		journal_file_nonfull_cond_signal(reader, diag);
 		mdhost_disconnect_request(c->host, NULL);
-
+	}
 	mdhost_put_peer(c->host, peer); /* decrement refcount */
 	gfmdc_journal_recv_end_signal(diag);
 #ifdef DEBUG_JOURNAL
@@ -1296,12 +1310,30 @@ gfmdc_journal_syncsend_thread(void *closure)
 static void
 gfmdc_wait_journal_recv_threads(const char *diag)
 {
+	struct timespec ts;
+	int in_time, timeout = gfarm_get_journal_sync_slave_timeout();
+	int timeout_max = 120;
+
+	gfarm_gettime(&ts);
+	ts.tv_sec += timeout;
 	gfarm_mutex_lock(&journal_sync_info.sync_mutex, diag,
 	    SYNC_MUTEX_DIAG);
-	while (journal_sync_info.nrecv_threads > 0)
-		gfarm_cond_wait(&journal_sync_info.sync_end_cond,
-		    &journal_sync_info.sync_mutex, diag,
+	while (journal_sync_info.nrecv_threads > 0) {
+		in_time = gfarm_cond_timedwait(
+		    &journal_sync_info.sync_end_cond,
+		    &journal_sync_info.sync_mutex, &ts, diag,
 		    SYNC_END_COND_DIAG);
+		if (in_time == 0) {
+			gflog_warning(GFARM_MSG_1004271,
+			    "journal response timeout (%d seconds), check "
+			    "synchronous slave servers", timeout);
+			timeout *= 2;
+			if (timeout > timeout_max)
+				timeout = timeout_max;
+			gfarm_gettime(&ts);
+			ts.tv_sec += timeout;
+		}
+	}
 	gfarm_mutex_unlock(&journal_sync_info.sync_mutex, diag,
 	    SYNC_MUTEX_DIAG);
 }
