@@ -803,11 +803,11 @@ process_reopen_file(struct process *process,
 					gfarm_error_string(e));
 				return (e);
 			}
-		} else if (!inode_schedule_confirm_for_write(fo, spool_host,
-		    &to_create)) {
-			gflog_debug(GFARM_MSG_1001631,
-				"file migrated");
-			return (GFARM_ERR_FILE_MIGRATED);
+		} else if ((e = inode_schedule_confirm_for_write(
+		    fo, spool_host, &to_create)) != GFARM_ERR_NO_ERROR) {
+			gflog_debug(GFARM_MSG_1001631, "%s",
+			    gfarm_error_string(e));
+			return (e);
 		}
 		if (to_create) {
 			e = inode_add_replica(fo->inode, spool_host, 1);
@@ -1089,8 +1089,20 @@ process_cksum_set(struct process *process, struct peer *peer, int fd,
 		    gfarm_error_string(e));
 		return (e);
 	}
-	return (file_opening_cksum_set(fo, cksum_type, cksum_len, cksum,
-	    flags, mtime));
+	e = file_opening_cksum_set(fo, cksum_type, cksum_len, cksum,
+	    flags, mtime);
+	if (e != GFARM_ERR_NO_ERROR && e != GFARM_ERR_EXPIRED) {
+		gflog_notice(GFARM_MSG_1004322,
+		    "(%s@%s) %s: inode %lld:%lld for %s-open%s: %s",
+		    peer_get_username(peer), peer_get_hostname(peer), diag,
+		    (long long)inode_get_number(fo->inode),
+		    (long long)inode_get_gen(fo->inode),
+		    accmode_to_string(fo->flag),
+		    (flags & GFM_PROTO_CKSUM_SET_REPORT_ONLY) != 0 ?
+		    " (report only)" : "",
+		    gfarm_error_string(e));
+	}
+	return (e);
 }
 
 gfarm_error_t
@@ -1371,6 +1383,13 @@ process_replica_added(struct process *process,
 			    (long long)inode_get_gen(fo->inode),
 			    gfarm_error_string(src_err),
 			    gfarm_error_string(dst_err));
+			/*
+			 * in this case, gfsd already knows the src_err
+			 * and dst_err, and will return the appropriate error
+			 * to the client.
+			 * thus, what gfmd should do here is to acknowledge
+			 * the report of the error.
+			 */
 			e = GFARM_ERR_NO_ERROR;
 		} else {
 			gflog_notice(GFARM_MSG_1002244,
@@ -1395,8 +1414,8 @@ process_replica_added(struct process *process,
 		    GFS_PROTO_REPLICATION_CKSUM_REQFLAG_INTERNAL_SUM_AVAIL
 		    ) == 0 && cksum_type != NULL && *cksum_type != '\0' &&
 		    cksum_len > 0) {
-			e = inode_cksum_set(fo->inode,
-			    cksum_type, cksum_len, cksum, cksum_result_flags);
+			e = inode_cksum_set(fo->inode, cksum_type, cksum_len,
+			    cksum, cksum_result_flags, 0);
 			if (e != GFARM_ERR_NO_ERROR)
 				gflog_notice(GFARM_MSG_1004221,
 				    "checksum error during replication of "
